@@ -273,7 +273,7 @@ def cmrealign(sequence, rRNA, refdir):
         f.close()
 
     mitfiimppath = os.path.join(
-        os.path.dirname(__file__), 'mitfi', 'parser', 'improve_infernal_structure.jar')
+        os.path.dirname(__file__), 'parser', 'improve_infernal_structure.jar')
 
     # improve structure
     proc = subprocess.Popen(["java", "-jar", mitfiimppath, tdir, "%s/cmrealign.stk" % (tdir)],
