@@ -1,24 +1,22 @@
 '''
 @author: M. Bernt
+
+This is a confidential release. Do not redistribute without 
+permission of the author (bernt@informatik.uni-leipzig.de).
 '''
+import logging
+from random import randint, shuffle
+from sys import stdout, stderr, exit
 
-from random import (
-    randint,
-    shuffle,
-)
-from sys import (
-    stderr,
-    stdout,
-)
-
-from Bio import (
-    Alphabet,
-    Data,
-    SeqIO,
-)
-from Bio.Alphabet import _verify_alphabet as verify_alphabet
+from Bio.Seq import Seq, translate
+from Bio import Alphabet
 from Bio.Alphabet.IUPAC import ambiguous_dna
-from Bio.Seq import Seq
+try:
+    from Bio.Alphabet import _verify_alphabet as verify_alphabet
+except:  # for old biopython
+    from Bio.utils import verify_alphabet
+from Bio import Data
+from Bio import SeqIO
 from Bio.SeqRecord import SeqRecord
 
 
@@ -48,7 +46,7 @@ class MultiFastaException(Exception):
 
 class AlphabetException(Exception):
     """
-    Exception to be raised when the alphabet does not match the sequence
+    Exception to be raised when the alphabet does not match the sequence 
     """
 
     def __init__(self, seq, alpha):
@@ -74,7 +72,7 @@ class sequence(Seq):
 
     def __init__(self, data, alphabet=Alphabet.IUPAC.IUPACAmbiguousDNA(), circular=False, upper=False):
         """
-        @param[in] upper if true: transform each letter to upper case, else: leave as is
+        @param[in] upper if true: transform each letter to upper case, else: leave as is 
         """
 
         data = data.strip().upper()
@@ -84,62 +82,12 @@ class sequence(Seq):
         if not verify_alphabet(self):
             raise AlphabetException(str(self), alphabet)
 
-    def __eq__(self, other):
-        """
-        check two sequences for equality
-        - same type (two sequences)
-        - same alphabet
-        - same circularity
-        - same data (ambiguities are checkes, i.e. N=A, ...)
-        """
 
-        if not isinstance(self, sequence) or not isinstance(other, sequence):
-            return False
-
-        if self.alphabet != other.alphabet:
-            return False
-
-        if self.circular != other.circular:
-            return False
-
-        if self.alphabet == Alphabet.IUPAC.ambiguous_dna or self.alphabet == Alphabet.IUPAC.unambiguous_dna:
-            ambiguous_nucleotide_values = Data.IUPACData.ambiguous_dna_values
-        elif self.alphabet == Alphabet.IUPAC.ambiguous_rna or self.alphabet == Alphabet.IUPAC.unambiguous_rna:
-            ambiguous_nucleotide_values = Data.IUPACData.ambiguous_rna_values
-        else:
-            return (str(self) == str(other))
-
-#        print self, other
-        for i in range(len(self)):
-            #            print self[i], other[i]
-            # print self[i], other[i], ambiguous_nucleotide_values[other[i] ],
-            # ambiguous_nucleotide_values[self[i] ]
-
-            try:
-                oa = set(ambiguous_nucleotide_values[other[i]])
-            except KeyError:
-                oa = set()
-
-            try:
-                sa = set(ambiguous_nucleotide_values[self[i]])
-            except KeyError:
-                sa = set()
-
-            if self[i] == other[i] or\
-                    (self[i] in oa) or (other[i] in sa)\
-                    or (not oa.isdisjoint(sa)):
-                continue
-            else:
-                #                print "RETURN FALSE"
-                return False
-#        print "RETURN TRUE"
-        return True
-
-    def __neq__(self, other):
-        """
-        check two sequences for inequality
-        """
-        return (not self == other)
+#    def __neq__( self, other ):
+#        """
+#        check two sequences for inequality
+#        """
+#        return ( not self == other )
 
     def __repr__(self):
         """
@@ -164,21 +112,21 @@ class sequence(Seq):
                                        mode,
                                        repr(self.alphabet))
 
-    # def __str__(self):
-    #     """
-    #     print sequence in fasta format
-    #     """
-    #     return "%s" %(self.__data)
+#     def __str__(self):
+#         """
+#         print sequence in fasta format
+#         """
+#         return "%s" %(self.__data)
 
     def __lshift__(self, other):
         """
         shift the sequence by other (int) to the left
         """
-        if isinstance(other, int):
-            raise TypeError("sequence shift with "
-                + repr(other) + " " + repr(type(other)) + " is impossible")
-        if not self.circular:
-            raise TypeError("sequence shift of linear sequence is impossible")
+        if (type(other) != type(0)):
+            raise TypeError, "sequence shift with " + \
+                repr(other) + " " + repr(type(other)) + " is impossible"
+        if (self.circular == False):
+            raise TypeError, "sequence shift of linear sequence is impossible"
 
         rdata = "".join([self[(i + other) % len(self)]
                          for i in range(len(self))])
@@ -188,11 +136,11 @@ class sequence(Seq):
         """
         shift the sequence by other (int) to the right
         """
-        if not isinstance(other, int):
-            raise TypeError("sequence shift with "
-                + repr(other) + " " + repr(type(other)) + " is impossible")
-        if not self.circular:
-            raise TypeError("sequence shift of linear sequence is impossible")
+        if (type(other) != type(0)):
+            raise TypeError, "sequence shift with " + \
+                repr(other) + " " + repr(type(other)) + " is impossible"
+        if (self.circular == False):
+            raise TypeError, "sequence shift of linear sequence is impossible"
 
         rdata = "".join([self[(i - other) % len(self)]
                          for i in range(len(self))])
@@ -201,8 +149,8 @@ class sequence(Seq):
     def dinucleotide_count(self, osb=True):
         """
         get the dinucleotide count of the sequence
-        @param osb if true consider only standard bases (ATCG)
-        @return a dictionary containing the counts
+        \param osb if true consider only standard bases (ATCG)
+        \return a dictionary containing the counts 
         """
         dn = {}
 
@@ -210,7 +158,7 @@ class sequence(Seq):
             for sb2 in Data.IUPACData.unambiguous_dna_letters:
                 dn[sb1 + sb2] = 0.0
 
-        for i in range(len(self)):
+        for i in xrange(len(self)):
             if not self.circular and (i + 1) >= len(self):
                 continue
 
@@ -225,21 +173,83 @@ class sequence(Seq):
     def dinucleotide_frequency(self, osb=True):
         """
         get the dinucleotide count of the sequence
-        @param osb if true consider only standard bases (ATCG)
-        @return a dictionary containing the frequencies
+        \param osb if true consider only standard bases (ATCG)
+        \return a dictionary containing the frequencies
         """
         dn = self.dinucleotide_count(osb)
         if self.circular:
-            sum = len(self)
+            sm = len(self)
         else:
-            sum = len(self) - 1
+            sm = len(self) - 1
 
         for k in dn:
 
-            dn[k] /= float(sum)
+            dn[k] /= float(sm)
 
         return dn
 
+    def isequal(self, other, maxac=None):
+        """
+        check two sequences for equality
+        - same type (two sequences)
+        - same alphabet
+        - same circularity
+        - same data (ambiguities are checkes, i.e. N=A, ...)
+        @param self seq
+        @param other another seq
+        @param maxac maximum number of allowed ambigous positions (in either sequence) 
+        """
+
+        if not isinstance(self, sequence) or not isinstance(other, sequence):
+            return False
+
+        if self.alphabet != other.alphabet:
+            return False
+
+        if self.circular != other.circular:
+            return False
+
+        if self.alphabet == Alphabet.IUPAC.ambiguous_dna or self.alphabet == Alphabet.IUPAC.unambiguous_dna:
+            ambiguous_nucleotide_values = Data.IUPACData.ambiguous_dna_values
+        elif self.alphabet == Alphabet.IUPAC.ambiguous_rna or self.alphabet == Alphabet.IUPAC.unambiguous_rna:
+            ambiguous_nucleotide_values = Data.IUPACData.ambiguous_rna_values
+        else:
+            return (str(self) == str(other))
+
+        ac = 0
+        for i in range(len(self)):
+            #            print self[i], other[i]
+            # print self[i], other[i], ambiguous_nucleotide_values[other[i] ],
+            # ambiguous_nucleotide_values[self[i] ]
+
+            try:
+                oa = set(ambiguous_nucleotide_values[other[i]])
+            except:
+                oa = set()
+
+            try:
+                sa = set(ambiguous_nucleotide_values[self[i]])
+            except:
+                sa = set()
+
+            if self[i] == other[i]:
+                continue
+            elif (self[i] in oa) or (other[i] in sa):
+                ac += 1
+                continue
+            elif (not oa.isdisjoint(sa)):
+                ac += 1
+                continue
+            else:
+                #                print "RETURN FALSE"
+                return False
+#         print "RETURN TRUE"
+
+        if maxac != None and ac > maxac:
+            return False
+        else:
+            return True
+
     def nucleotide_count(self, osb=True):
         """
         count the number of occurences of a,t,c,g,...
@@ -250,7 +260,7 @@ class sequence(Seq):
         for sb in Data.IUPACData.unambiguous_dna_letters:
             nc[sb] = 0.0
 
-        for i in range(len(self)):
+        for i in xrange(len(self)):
             if osb and (self[i] not in Data.IUPACData.unambiguous_dna_letters):
                 continue
             if not self[i] in nc:
@@ -264,10 +274,10 @@ class sequence(Seq):
         get the frequency of the nucleotides in the sequence
         """
         nc = self.nucleotide_count(osb)
-        sum = len(self)
+        sm = len(self)
         for k in nc:
             try:
-                nc[k] /= float(sum)
+                nc[k] /= float(sm)
             except ZeroDivisionError:
                 nc[k] = 0.0
 
@@ -275,7 +285,7 @@ class sequence(Seq):
 
     def shuffle(self):
         """
-        random shuffle the sequence
+        random shuffle the sequence 
         """
         tdata = [x for x in self.data]
         shuffle(tdata)
@@ -287,8 +297,8 @@ class sequence(Seq):
         note: subsequences are always linear
 
         @param start start index, note counting starts at 0
-        @param stop end index, note element at the stop index is included in the sequence
-        @param strand +1/-1 get the reverse complement of the sequence if -1 and the sequence if 1
+        @param stop end index, note element at the stop index is included in the sequence  
+        @param strand +1/-1 get the reverse complement of the sequence if -1 and the sequence if 1 
         """
 
         if strand != 1 and strand != -1:
@@ -296,11 +306,19 @@ class sequence(Seq):
             raise Exception("StrandError", "strand is", strand)
 
         if not self.circular and (start < 0 or stop < 0):
-            stderr.write("error: [%d,%d] of linear sequence\n" % (start, stop))
+            raise Exception(
+                "error: [%d,%d] of linear sequence\n" % (start, stop))
 
-        while start < 0 or stop < 0:
-            start += len(self)
-            stop += len(self)
+        if not self.circular and (start >= len(self) or stop >= len(self)):
+            raise Exception(
+                "error: [%d,%d] of linear sequence of length %d\n" % (start, stop, len(self)))
+
+        if self.circular:
+            while start < 0 or stop < 0:
+                start += len(self)
+                stop += len(self)
+            start %= len(self)
+            stop %= len(self)
 
         if self.circular and start > len(self):
             start %= len(self)
@@ -317,7 +335,8 @@ class sequence(Seq):
                     str(self[start:] + self[:stop + 1]), self.alphabet, circular=False)
 #                seq = self.__class__( self.data[start:] + self.data[:stop + 1], self.alphabet, circular = False )
             else:
-                stdout.write("warning: [%d,%d] of linear sequence\n" % (start, stop))
+                raise Exception(
+                    "error: [%d,%d] of linear sequence\n" % (start, stop))
 
         if strand == -1:
             seq = seq.reverse_complement()
@@ -326,14 +345,14 @@ class sequence(Seq):
 
         return seq
 
-    def start_stop_subsequence(self, transl_table, max=False):
+    def start_stop_subsequence(self, transl_table, mx=False):
         """
-        get (start,stop, strand) tuples of the sequence such that at start is a
+        get (start,stop, strand) tuples of the sequence such that at start is a 
         start codon and at end a end codon
-        @param transl_table the id of the translation table
-        @param max only return maximal sequences, i.e. if there are two sequences
+        \param transl_table the id of the translation table
+        \param mx only return maximal sequences, i.e. if there are two sequences 
         (s1,e) and (s2,e) with s1<s2 then only (s1,e) will be returned
-        @todo subsequences crossing the 0 .. attention the reading frame calculations
+        \todo subsequences crossing the 0 .. attention the reading frame calculations 
         get tricky
         """
 
@@ -354,13 +373,13 @@ class sequence(Seq):
             elif subs in stop_codons:
                 for s in stack[i % 3]:
                     ret.append((s, (i + 2) % len(self), 1))
-                    if max:
+                    if mx:
                         break
 
                 stack[i % 3] = []
 
         # first -strand
-        if not self.circular:
+        if self.circular == False:
             return ret
 
         stack = [[], [], []]
@@ -373,30 +392,42 @@ class sequence(Seq):
             if subs in stop_codons:
                 for s in stack[i % 3]:
                     ret.append(((i + 2) % len(self), s, -1))
-                    if max:
+                    if mx:
                         break
                 stack[i % 3] = []
 
         return ret
 
+    def translate(self, table='Standard', stop_symbol='*', to_stop=False, cds=False, gap=None):
+        """
+        little helper to translate the nucleotide sequence
+        adds N until multiple of three and calls biopythons translate foo
+        """
+        seq = self._data
+        while len(seq) % 3 != 0:
+            seq += "N"
+        return translate(seq, table, stop_symbol, to_stop, cds)
+
     def deambig(self):
         """
-        return a list of sequences
+        return a list of sequences 
         """
         # print "deambig"
         if self.alphabet == Alphabet.IUPAC.ambiguous_dna or self.alphabet == Alphabet.IUPAC.unambiguous_dna:
             # print "dna"
             ambiguous_nucleotide_values = Data.IUPACData.ambiguous_dna_values
+            # unambiguous_alph = Alphabet.IUPAC.unambiguous_dna
         elif self.alphabet == Alphabet.IUPAC.ambiguous_rna or self.alphabet == Alphabet.IUPAC.unambiguous_rna:
             # print "rna"
             ambiguous_nucleotide_values = Data.IUPACData.ambiguous_rna_values
+            # unambiguous_alph = Alphabet.IUPAC.unambiguous_rna
         else:
             # print self.alphabet
-            return [sequence(str(self), self.alphabet, False)]
+            return [str(self)]
         # print
         # print(self.data)
 
-        c1, c2, c3 = self.data
+        c1, c2, c3 = self._data
         x1 = ambiguous_nucleotide_values[c1]
         x2 = ambiguous_nucleotide_values[c2]
         x3 = ambiguous_nucleotide_values[c3]
@@ -418,12 +449,12 @@ class randsequence(sequence):
     def __init__(self, length, alphabet=Alphabet.IUPAC.IUPACUnambiguousDNA(), circular=False):
         """
         generate a random sequence of a certain alphabet
-        @param length how long should the sequence be
-        @param alphabet which alphabet
-        @param circular init as circular
+        \param length how long should the sequence be
+        \param alphabet which alphabet 
+        \param circular init as circular
         """
         data = []
-        for i in range(length):
+        for i in xrange(length):
             data.append(
                 alphabet.letters[randint(0, len(alphabet.letters) - 1)])
         data = "".join(data).strip().upper()
@@ -461,7 +492,7 @@ def sequences_fromfile(fname, alphabet=ambiguous_dna, circular=False):
 
 def sequence_info_fromfilehandle(handle, alphabet=ambiguous_dna, circular=False):
 
-    seq = []
+    seqlist = []
     for seq_record in SeqIO.parse(handle, "fasta"):
         #        raise Exception( str( seq_record ) )
         #        print dir(seq_record)
@@ -472,13 +503,12 @@ def sequence_info_fromfilehandle(handle, alphabet=ambiguous_dna, circular=False)
         #        print 'format', seq_record.format
         #        print 'id', seq_record.id
         #        print 'name', seq_record.name
-        seq.append({'name': seq_record.name.strip(),
-                    'description': seq_record.description.strip(),
-                    'sequence': sequence(str(seq_record.seq), alphabet, circular=circular),
-                    'id': seq_record.id})
-        # print seq[-1]
+        seqlist.append({'name': seq_record.name.strip(),
+                        'description': seq_record.description.strip(),
+                        'sequence': sequence(str(seq_record.seq), alphabet, circular=circular),
+                        'id': seq_record.id})
 
-    return seq
+    return seqlist
 
 
 def sequence_info_fromfile(fname, alphabet=ambiguous_dna, circular=False):
@@ -487,10 +517,9 @@ def sequence_info_fromfile(fname, alphabet=ambiguous_dna, circular=False):
     """
 
     handle = open(fname, "r")
-    seq = sequence_info_fromfilehandle(handle, alphabet, circular)
+    seqlist = sequence_info_fromfilehandle(handle, alphabet, circular)
     handle.close()
-
-    return seq
+    return seqlist
 
 
 def seqlistmaker(sequence, start, stop, strand, acc, name="", code=None, seqlist=[]):
@@ -500,7 +529,7 @@ def seqlistmaker(sequence, start, stop, strand, acc, name="", code=None, seqlist
     @param stop the stop of the part
     @param acc the acc of the species
     @param name the name of the part if ther is a name.
-    @param code translate with code, do not translate if None
+    @param code translate with code, do not translate if None 
     @param seqlist A list of sequences, online needet if this sequenz is only a part of bigger list.
 
     This method cut a part of a big sequence and append it in a list.
@@ -527,8 +556,8 @@ def seqlistmaker(sequence, start, stop, strand, acc, name="", code=None, seqlist
     # set the subsequnc, need a Bio.Seq.Seq format
     mito_frag = sequence.subseq(start, stop, int(strand))
 
-    if code is not None:
-        mito_frag = mito_frag.translate(table=code)
+    if code != None:
+        mito_frag = Seq(mito_frag.translate(table=code))
 
     # Set record with header
     record = SeqRecord(mito_frag, out, '', '')
@@ -551,11 +580,9 @@ def fastawriter(featurelist, sequence, code, acc, outputtype, outfile=None, mode
                      strand=feat.strand, name=feat.outputname(), code=code, seqlist=seqlist)
 
     if isinstance(outfile, str):
-        file = open(outfile, mode)
-        SeqIO.write(seqlist, file, "fasta")
-        file.close()
-
-    elif outfile is None:
+        with open(outfile, mode) as f:
+            SeqIO.write(seqlist, f, "fasta")
+    elif outfile == None:
         SeqIO.write(seqlist, stdout, "fasta")
     else:
         SeqIO.write(seqlist, outfile, "fasta")
