@@ -1,17 +1,17 @@
+from __future__ import print_function
 import argparse
 import cPickle
 import itertools
 import logging
-import multiprocessing
 import os.path
 import random
-import re
 import sys
-
+import re
+import multiprocessing
 import numpy
 
-from mitos import mitofile
-from mitos.rna import forester
+import rna.forester
+import mitofile
 
 
 def apply_statistics(allocomp, allocompdist, isocomp, isocompdist):
@@ -81,7 +81,7 @@ def apply_statistics(allocomp, allocompdist, isocomp, isocompdist):
                 # that are <= the value; thus large ranks are of interest
                 # (which is as in the z-score)
                 rank = len(
-                    [x for x in allocompdist[name][name2] if x < allocomp[acc][name][i]["sim"]])
+                    [x for x in allocompdist[name][name2] if(x < allocomp[acc][name][i]["sim"])])
                 # norm
 
                 allocomp[acc][name][i]["rank"] = float(
@@ -90,13 +90,13 @@ def apply_statistics(allocomp, allocompdist, isocomp, isocompdist):
 
 def compare(p):
     """
-    execute a comparison function for a pair of
-    list of sequences and list of structures
+    execute a comparison function for a pair of 
+    list of sequences and list of structures 
     """
-    # print p[0], p[1], forester.RNAforester(sequences = p[0],structures =
+    # print p[0], p[1], rna.forester.RNAforester(sequences = p[0],structures =
     # p[1], r = True )
 
-    return forester.RNAforester(
+    return rna.forester.RNAforester(
         sequences=p[0],
         structures=p[1], r=True)
 
@@ -117,7 +117,7 @@ def compareaa(lst):
             structures.append(lst[i].structure)
             structures.append(lst[j].structure)
 
-    scores = forester.RNAforester(sequences, structures, r=True)
+    scores = rna.forester.RNAforester(sequences, structures, r=True)
     scoremat = []
     x = 0
     for i in range(len(lst)):
@@ -166,7 +166,7 @@ def get_alloacceptorhist(data, fname):
             f.write(str(feat.name) + "\n")
             try:
                 hist[str(feat.name)] += 1
-            except KeyError:
+            except:
                 hist[str(feat.name)] = 1
 
     f.close()
@@ -178,10 +178,10 @@ def get_alloacceptorhist(data, fname):
 def get_allocomparisons(data, threads):
     """
     get the comparisions of the tRNAs in the same species
-    (the name is due that originally only those with different
+    (the name is due that originally only those with different 
     alloacceptor have been generated)
 
-    @param data the data
+    @param data the data 
     @param threads number of threads to use
     """
 
@@ -194,16 +194,16 @@ def get_allocomparisons(data, threads):
     x = 0
 
     for acc in data:
-        print(acc)
+        # print(acc)
         sys.stderr.write("allo %s %d/%d\r" % (acc, x + 1, len(data)))
 
-        if acc not in allocomp:
+        if not acc in allocomp:
             allocomp[acc] = {}
 
 #        dmatrix = compareaa( data[acc] )
         for i in range(len(data[acc])):
             f1 = data[acc][i]
-            if f1.name not in allocomp[acc]:
+            if not f1.name in allocomp[acc]:
                 allocomp[acc][f1.name] = []
             # compare to different alloacceptor in the same species
             for j in range(len(data[acc])):
@@ -239,7 +239,7 @@ def get_allocompdist(allocomp, fname):  # i'm here now
     allocompdist = {}
     for acc in allocomp:
         for name in allocomp[acc]:
-            if name not in allocompdist:
+            if not name in allocompdist:
                 allocompdist[name] = {}
 
             for i in range(len(allocomp[acc][name])):
@@ -266,11 +266,11 @@ def get_isocomp(data, tax, threads, mult):
     """
     isocomp = {}
 
-    ln = 1
+    l = 1
     for acc in data:
 
-        sys.stderr.write("iso %s %d/%d\r" % (acc, ln, len(data)))
-        if acc not in isocomp:
+        sys.stderr.write("iso %s %d/%d\r" % (acc, l, len(data)))
+        if not acc in isocomp:
             isocomp[acc] = {}
 
         # get trnas with the same anticodon
@@ -280,7 +280,7 @@ def get_isocomp(data, tax, threads, mult):
         st = sorted_taxonomy(acc, tax)
         for ac in anticodons:
 
-            if ac not in isocomp[acc]:
+            if not ac in isocomp[acc]:
                 isocomp[acc][ac] = []
 
             cand = {}
@@ -327,7 +327,7 @@ def get_isocomp(data, tax, threads, mult):
                     for c2 in cand[accession2]:
                         isocomp[acc][ac].append(
                             {"t1": c1, "t2": c2, "sim": 0, "wrt": accession2})
-        ln += 1
+        l += 1
 
     # compile comparisons to execute
     comparisons = []
@@ -352,7 +352,7 @@ def get_isocomp(data, tax, threads, mult):
             if not mult:
                 continue
 
-                print("Here")
+            # print("Here")
             # ONLY REMOVE ONE PER TRNA
             t1s = sorted([z['t1'] for z in isocomp[acc][ac]])
             t1s = [key for key, _ in itertools.groupby(t1s)]
@@ -361,7 +361,7 @@ def get_isocomp(data, tax, threads, mult):
             isocomp[acc][ac] = []
 
             for t in t1s:
-                best = [z for z in tmp if z['t1'] == t]
+                best = [z for z in tmp if(z['t1'] == t)]
                 best.sort(key=lambda k: k['sim'])
                 isocomp[acc][ac].append(best[0])
 
@@ -373,7 +373,7 @@ def get_isocompdist(isocomp, fname):
     f = open(fname, "w")
     for acc in isocomp:
         for ac in isocomp[acc]:
-            if ac not in isocompdist:
+            if not ac in isocompdist:
                 isocompdist[ac] = []
 
             for i in range(len(isocomp[acc][ac])):
@@ -385,7 +385,7 @@ def get_isocompdist(isocomp, fname):
                 f.write("%s %f %d\n" % (ac, isocomp[acc][ac][i]['sim'], copy))
                 isocompdist[ac].append(isocomp[acc][ac][i]['sim'])
 
-    for ac in isocompdist.keys():
+    for ac in list(isocompdist.keys()):
         if len(isocompdist[ac]) > 0:
             isocompdist[ac].sort()
         else:
@@ -404,7 +404,7 @@ def get_isoacceptorhist(data, fname):
             f.write(str(feat.anticodon) + "\n")
             try:
                 hist[str(feat.anticodon)] += 1
-            except KeyError:
+            except:
                 hist[str(feat.anticodon)] = 1
     f.close()
 
@@ -413,13 +413,13 @@ def get_isoacceptorhist(data, fname):
 
 def get_results(data, allocomp, allocompdist, isocomp, isocompdist, stat, fname):
     """
-    @param data
+    @param data 
     @param allocomp
     @param allocompdist
-    @param isocomp
+    @param isocomp 
     @param isocompdist
     @param stat statistics to use (z/rank/sim)
-    fname
+    fname 
     """
     # print "Allocomp", allocomp, "End-Allocomp "+stat+"\n \n \n"
     # print "Allocompdist", allocompdist, "End-Allocomp dist \n \n \n"
@@ -430,9 +430,9 @@ def get_results(data, allocomp, allocompdist, isocomp, isocompdist, stat, fname)
     f = open(fname, "w")
 
     stop = False
-    ln = 0
+    l = 0
     for acc in data:
-        sys.stderr.write("%s %d/%d\r" % (acc, ln + 1, len(data)))
+        sys.stderr.write("%s %d/%d\r" % (acc, l + 1, len(data)))
 
         # if acc != "NC_008140":
         for i in range(len(data[acc])):
@@ -461,17 +461,17 @@ def get_results(data, allocomp, allocompdist, isocomp, isocompdist, stat, fname)
             for acmp in allocomp[acc][data[acc][i].name]:
                 # fraction of the allo-comparisons that are more similar than
                 # this pair
-                if maxcmpap is None or acmp[stat] > maxcmpap[stat]:
+                if maxcmpap == None or acmp[stat] > maxcmpap[stat]:
                     maxcmpap = acmp
 
             # determine min similarity of the tRNA with another with the same
             # isoacc
             comparision = [x for x in isocomp[acc][str(data[acc][i].anticodon)]
-                           if x['t1'] == data[acc][i]]
+                           if(x['t1'] == data[acc][i])]
 
             mincmpip = None
             for c in comparision:
-                if mincmpip is None or c[stat] < mincmpip[stat]:
+                if mincmpip == None or c[stat] < mincmpip[stat]:
                     mincmpip = c
 
             try:
@@ -481,7 +481,7 @@ def get_results(data, allocomp, allocompdist, isocomp, isocompdist, stat, fname)
                                                                           maxcmpap['t1'].name, maxcmpap['t2'].name, maxcmpap[
                                                                               'sim'], maxcmpap['rank'], maxcmpap['z'],
                                                                           mincmpip['wrt'], str(mincmpip['t2'].anticodon), mincmpip['sim'], mincmpip['rank'], mincmpip['z']))
-            except Exception:
+            except:
                 sys.stderr.write("%s %s %s %s\n" % (
                     acc, data[acc][i].name, data[acc][i].anticodon, repr(data[acc][i])))
                 print(mincmpip)
@@ -496,11 +496,11 @@ def get_results(data, allocomp, allocompdist, isocomp, isocompdist, stat, fname)
                 stop = True
                 break
             # else:
-            #   continue
+             #   continue
 
-        if stop is True:
+        if stop == True:
             break
-        ln += 1
+        l += 1
 
 
 #            .append( { "t1": c1, "t2":c2, "sim":compare( c1, c2 ), "wrt": accession2 } )
@@ -534,7 +534,7 @@ def load_data(dirs):
     # random bases(ACGTU)
     for dr in dirs:
         # if not os.listdir(dr) == []:
-        #    continue
+            # continue
         if not os.path.isdir(dr):
             sys.stderr.write("skipping non dir %s\n" % dr)
             continue
@@ -554,13 +554,13 @@ def load_data(dirs):
             continue
 
         tmp = [x for x in features if (
-            x.type == "tRNA" and x.anticodon is None)]
+            x.type == "tRNA" and x.anticodon == None)]
         for t in tmp:
             logging.warning("remove degenerated tRNA: %s %s" %
                             (gb.accession, t.name))
 
         data[gb.accession] = [
-            x for x in features if (x.type == "tRNA" and x.anticodon is not None)]
+            x for x in features if (x.type == "tRNA" and x.anticodon != None)]
 
         for i in range(len(data[gb.accession])):
             while 1:
@@ -594,29 +594,29 @@ def load_taxonomy(taxfname):
 def sorted_taxonomy(acc, tax):
     """
     given an accession a and the taxonomy lists for a set of accessions S
-    including the accession a itself return a list of the accessions S
-    sorted by their distance to a. The distance is the distance in the tree.
-    Let a and b two accessions and T_a and T_b their accesion lists, and
+    including the accession a itself return a list of the accessions S 
+    sorted by their distance to a. The distance is the distance in the tree. 
+    Let a and b two accessions and T_a and T_b their accesion lists, and 
     C be the common prefix of T_a and T_b. Then the distance is computed as
-    (|T_a|-|C|)+(|T_b|-|C|).
+    (|T_a|-|C|)+(|T_b|-|C|).  
 
     @param acc accession
     @param tax taxonomy lists for all accessions given as dict
-    @return sorted list of accessions
+    @return sorted list of accessions  
     """
     st = {}
     for t in tax:
-        ln = len_common_prefix(tax[acc], tax[t])
-        d = (len(tax[acc]) - ln) + (len(tax[t]) - ln)
+        l = len_common_prefix(tax[acc], tax[t])
+        d = (len(tax[acc]) - l) + (len(tax[t]) - l)
 
-        if d not in st:
+        if not d in st:
             st[d] = []
 
         st[d].append(t)
 
     return st
 
-
+#    return dist
 if __name__ == '__main__':
     usage = "usage: %prog dirs"
     parser = argparse.ArgumentParser(description=usage)
@@ -703,7 +703,8 @@ if __name__ == '__main__':
             #            if str( c["t1"].anticodon ) != acod:
             #                continue
 
-            print(c["t1"].outputname(), c["t2"].outputname(), c["sim"], c["rank"], c["z"])
+            print(c["t1"].outputname(), c["t2"].outputname(),
+                  c["sim"], c["rank"], c["z"])
 
         for feat in data[acc]:
             if name != feat.name:
@@ -711,7 +712,8 @@ if __name__ == '__main__':
 
             ac = str(feat.anticodon)
             for c in sorted(isocomp[acc][ac], key=lambda k: k['sim']):
-                print(c["wrt"], c["t1"].outputname(), c["sim"], c["rank"], c["z"])
+                print(
+                    c["wrt"], c["t1"].outputname(), c["sim"], c["rank"], c["z"])
 
 
 #    sys.exit()
