@@ -860,7 +860,7 @@ if __name__ == '__main__':
     # parameter from cmsearch function
 
     import argparse
-    usage = "usage: %prog [options]"
+    usage = "%(prog)s [options]"
     parser = argparse.ArgumentParser(prog="runmitos.py", usage=usage)
 
     group = parser.add_argument_group("mandatory options")
@@ -943,7 +943,7 @@ if __name__ == '__main__':
         "--debug", action="store_true", default=False, help="print debug output")
     group.add_argument("--json", default=None,
                        help="a JSON file with parameters. then outdir is the only other argument needed.")
-    parser.add_argument('--version', action='version', version='%(prog)s {version}'.format(version=__version__))
+    group.add_argument('--version', action='version', version='%(prog)s {version}'.format(version=__version__))
     args = parser.parse_args()
 
     if args.json is not None:
